# pyright: reportUnusedImport=none
from .auth_context import get_token
from .jwt_auth import JwtAuth
from .models import AuthConfig, Token, TokenRole, UserInfoLogRecord
from .user_name_logging_filter import UserNameLoggingFilter



from typing import List

import pytest
from fastapi import HTTPException
from jose import JWTError
from pytest import MonkeyPatch

from sag_py_auth.jwt_auth import JwtAuth
from sag_py_auth.models import AuthConfig, Token, TokenRole
from sag_py_auth.token_types import TokenDict

from .helpers import get_token_dict


def verify_and_decode_token_mock(auth_config: AuthConfig, token_string: str) -> TokenDict:
    if token_string == "validToken":
        return get_token_dict(None, None)
    else:
        raise JWTError(f"Invalid payload string: {token_string}")


def test__verify_and_decode_token__with_valid_token(monkeypatch: MonkeyPatch) -> None:
    # Arrange
    auth_config = AuthConfig("https://authserver.com/auth/realms/projectName", "audienceOne")
    required_roles: List[TokenRole] = [TokenRole("clientOne", "clientOneRoleOne")]
    required_realm_roles: List[str] = ["realmRoleOne"]

    monkeypatch.setattr("sag_py_auth.jwt_auth.verify_and_decode_token", verify_and_decode_token_mock)

    jwt = JwtAuth(auth_config, required_roles, required_realm_roles)

    # Act
    actual: Token = jwt._verify_and_decode_token("validToken")

    # Asserrt
    assert actual.get_field_value("typ") == "Bearer"
    assert actual.get_field_value("azp") == "public-project-swagger"


def test__verify_and_decode_token__with_invalid_token(monkeypatch: MonkeyPatch) -> None:
    with pytest.raises(HTTPException) as exception:
        # Arrange
        auth_config = AuthConfig("https://authserver.com/auth/realms/projectName", "audienceOne")
        required_roles: List[TokenRole] = [TokenRole("clientOne", "clientOneRoleOne")]
        required_realm_roles: List[str] = ["realmRoleOne"]

        monkeypatch.setattr("sag_py_auth.jwt_auth.verify_and_decode_token", verify_and_decode_token_mock)

        jwt = JwtAuth(auth_config, required_roles, required_realm_roles)

        # Act
        jwt._verify_and_decode_token("invalidToken")

    # Assert
    assert exception.value.status_code == 401
    assert exception.value.detail == "Invalid token."



from contextvars import ContextVar
from typing import Optional

from sag_py_auth.models import Token

token: ContextVar[Optional[Token]] = ContextVar("token", default=None)


def get_token() -> Optional[Token]:
    """Gets the context local token. See library contextvars for details.

    Returns: The token
    """
    return token.get(None)


def set_token(token_to_set: Token) -> None:
    """Sets the context local token. See library contextvars for details."""
    token.set(token_to_set)



from typing import Dict, List, Optional

import pytest
from fastapi import HTTPException

from sag_py_auth.jwt_auth import JwtAuth
from sag_py_auth.models import AuthConfig, Token

from .helpers import get_token


def test__verify_realm_roles__has_multiple() -> None:
    # Arrange
    jwt_auth = JwtAuth(
        AuthConfig("https://authserver.com/auth/realms/projectName", "audienceOne"),
        None,
        ["realmRoleOne", "realmRoleTwo"],
    )

    realm_access: Dict[str, List[str]] = {"roles": ["realmRoleOne", "realmRoleTwo"]}

    token: Optional[Token] = get_token(realm_access, None)

    # Act
    try:
        jwt_auth._verify_realm_roles(token)
    except Exception:
        pytest.fail("No exception expected if the user has all realm roles")


def test__verify_realm_roles__requires_none() -> None:
    # Arrange
    jwt_auth = JwtAuth(AuthConfig("https://authserver.com/auth/realms/projectName", "audienceOne"), None, None)

    realm_access: Dict[str, List[str]] = {"roles": ["realmRoleOne", "realmRoleTwo"]}

    token: Token = get_token(realm_access, None)

    # Act
    try:
        jwt_auth._verify_realm_roles(token)
    except Exception:
        pytest.fail("No exception expected if no roles are required")


def test__verify_realm_roles__requires_empty() -> None:
    # Arrange
    jwt_auth = JwtAuth(AuthConfig("https://authserver.com/auth/realms/projectName", "audienceOne"), None, [])

    realm_access: Dict[str, List[str]] = {"roles": ["realmRoleOne", "realmRoleTwo"]}

    token: Token = get_token(realm_access, None)

    # Act
    try:
        jwt_auth._verify_realm_roles(token)
    except Exception:
        pytest.fail("No exception expected if no roles are required")


def test__verify_realm_roles__missing_realm_role() -> None:
    with pytest.raises(HTTPException) as exception:
        # Arrange
        jwt_auth = JwtAuth(
            AuthConfig("https://authserver.com/auth/realms/projectName", "audienceOne"),
            None,
            ["realmRoleOne", "realmRoleTwo"],
        )

        realm_access: Dict[str, List[str]] = {"roles": ["realmRoleOne"]}

        token: Token = get_token(realm_access, None)

        # Act
        jwt_auth._verify_realm_roles(token)

    # Assert
    assert exception.value.status_code == 403
    assert exception.value.detail == "Missing realm role."


def test__verify_realm_roles__token_with_empty_realm_roles() -> None:
    with pytest.raises(HTTPException) as exception:
        # Arrange
        jwt_auth = JwtAuth(
            AuthConfig("https://authserver.com/auth/realms/projectName", "audienceOne"),
            None,
            ["realmRoleOne", "realmRoleTwo"],
        )

        token: Token = get_token({}, None)

        # Act
        jwt_auth._verify_realm_roles(token)

    # Assert
    assert exception.value.status_code == 403
    assert exception.value.detail == "Missing realm role."


def test__verify_realm_roles__token_without_realm_roles() -> None:
    with pytest.raises(HTTPException) as exception:
        # Arrange
        jwt_auth = JwtAuth(
            AuthConfig("https://authserver.com/auth/realms/projectName", "audienceOne"),
            None,
            ["realmRoleOne", "realmRoleTwo"],
        )

        token: Token = get_token(None, None)

        # Act
        jwt_auth._verify_realm_roles(token)

    # Assert
    assert exception.value.status_code == 403
    assert exception.value.detail == "Missing realm role."



import pytest

from sag_py_auth.utils import validate_url


@pytest.mark.parametrize(
    "url",
    [
        "https://google.de",
        "http://google.de",
        "https://google.de/",
        "https://google.de/test/test2",
        "https://localhost",
    ],
)
def test__validate_url__valid_urls(url: str) -> None:
    # Act
    actual: bool = validate_url(url)

    # Assert
    assert actual


@pytest.mark.parametrize("url", ["google.de", "google", "/test/test", "", None])
def test__validate_url__invalid_urls(url: str) -> None:
    # Act
    actual: bool = validate_url(url)

    # Assert
    assert not actual



from logging import Filter, LogRecord
from typing import Optional

from sag_py_auth.auth_context import get_token
from sag_py_auth.models import Token


class UserNameLoggingFilter(Filter):
    """Register this filter to get a field user_name and authorized_party in log entries"""

    def __init__(self, name: str = "") -> None:
        super().__init__(name=name)

    def filter(self, record: LogRecord) -> bool:
        token: Optional[Token] = get_token()
        user_name: str = token.get_field_value("preferred_username") if token else ""
        authorized_party: str = token.get_field_value("azp") if token else ""

        if user_name:
            record.user_name = user_name

        if authorized_party:
            record.authorized_party = authorized_party

        return True



from typing import Optional

from sag_py_auth.auth_context import get_token as get_token_from_context
from sag_py_auth.auth_context import set_token as set_token_to_context
from sag_py_auth.models import Token

from .helpers import get_token as get_test_token


# Note: This test is not entirely independent of the other test
# because they run in the same context and therefor share a context.
# That's why the "not_set_token" test has to run before the "with_previously_set_token"
# Furthermore other tests that run the __call__ method of jwt_auth could break that one.
def test__get_token__not_set_token() -> None:
    # Act
    actual: Optional[Token] = get_token_from_context()

    assert actual is None


def test__get_token__with_previously_set_token() -> None:
    # Arrange
    token: Token = get_test_token(None, None)

    # Act
    set_token_to_context(token)
    actual: Optional[Token] = get_token_from_context()

    assert actual == token



from typing import Any, Dict

from mock import Mock
from pytest import MonkeyPatch
from requests import Response

from sag_py_auth.token_decoder import _get_token_jwk
from sag_py_auth.token_types import JwkDict


def get_mock(url: str, headers: Dict[str, str], timeout: int) -> Response:
    if (
        url == "https://authserver.com/auth/realms/projectName/protocol/openid-connect/certs"
        and headers["content-type"] == "application/json"
    ):
        return Mock(
            status_code=200,
            json=lambda: {
                "keys": [
                    {
                        "kid": "123456",
                        "kty": "RSA",
                        "alg": "RS256",
                    },
                    {
                        "kid": "654321",
                        "kty": "RSA",
                        "alg": "RS256",
                    },
                ]
            },
        )

    return Response()


def get_unverified_header_mock(token_string: str) -> Dict[str, Any]:
    return {"kid": "654321"} if token_string == "validTokenString" else {}


def test__get_token_jwk(monkeypatch: MonkeyPatch) -> None:
    # Arrange
    monkeypatch.setattr("sag_py_auth.token_decoder.requests.get", get_mock)
    monkeypatch.setattr("sag_py_auth.token_decoder.jwt.get_unverified_header", get_unverified_header_mock)

    # Act
    actual: JwkDict = _get_token_jwk("https://authserver.com/auth/realms/projectName", "validTokenString")

    # Assert
    assert actual == {"kid": "654321", "kty": "RSA", "alg": "RS256"}



from typing import Dict, List

import pytest
from fastapi import HTTPException

from sag_py_auth.jwt_auth import JwtAuth
from sag_py_auth.models import AuthConfig, Token, TokenRole

from .helpers import get_token


def test__verify_roles__has_multiple() -> None:
    # Arrange
    jwt_auth = JwtAuth(
        AuthConfig("https://authserver.com/auth/realms/projectName", "audienceOne"),
        [TokenRole("clientOne", "clientOneRoleOne"), TokenRole("clientTwo", "clientTwoRoleTwo")],
        None,
    )

    resource_access: Dict[str, Dict[str, List[str]]] = {
        "clientOne": {"roles": ["clientOneRoleOne", "clientOneRoleTwo"]},
        "clientTwo": {"roles": ["clientTwoRoleOne", "clientTwoRoleTwo"]},
    }

    token: Token = get_token(None, resource_access)

    # Act
    try:
        jwt_auth._verify_roles(token)
    except Exception:
        pytest.fail("No exception expected if the user has all roles")


def test__verify_roles__requires_none() -> None:
    # Arrange
    jwt_auth = JwtAuth(AuthConfig("https://authserver.com/auth/realms/projectName", "audienceOne"), None, None)

    resource_access: Dict[str, Dict[str, List[str]]] = {
        "clientOne": {"roles": ["clientOneRoleOne", "clientOneRoleTwo"]},
        "clientTwo": {"roles": ["clientTwoRoleOne", "clientTwoRoleTwo"]},
    }

    token: Token = get_token(None, resource_access)

    # Act
    try:
        jwt_auth._verify_roles(token)
    except Exception:
        pytest.fail("No exception expected if the user requires no roles")


def test__verify_roles__requires_empty() -> None:
    # Arrange
    jwt_auth = JwtAuth(AuthConfig("https://authserver.com/auth/realms/projectName", "audienceOne"), [], None)

    resource_access: Dict[str, Dict[str, List[str]]] = {
        "clientOne": {"roles": ["clientOneRoleOne", "clientOneRoleTwo"]},
        "clientTwo": {"roles": ["clientTwoRoleOne", "clientTwoRoleTwo"]},
    }

    token: Token = get_token(None, resource_access)

    # Act
    try:
        jwt_auth._verify_roles(token)
    except Exception:
        pytest.fail("No exception expected if the user requires no roles")


def test__verify_roles__missing_role_of_existing_client() -> None:
    with pytest.raises(HTTPException) as exception:
        # Arrange
        jwt_auth = JwtAuth(
            AuthConfig("https://authserver.com/auth/realms/projectName", "audienceOne"),
            [
                TokenRole("clientOne", "clientOneRoleOne"),
                TokenRole("clientTwo", "clientTwoRoleTwo"),
                TokenRole("clientTwo", "missingRole"),
            ],
            None,
        )

        resource_access: Dict[str, Dict[str, List[str]]] = {
            "clientOne": {"roles": ["clientOneRoleOne", "clientOneRoleTwo"]},
            "clientTwo": {"roles": ["clientTwoRoleOne", "clientTwoRoleTwo"]},
        }

        token: Token = get_token(None, resource_access)

        # Act
        jwt_auth._verify_roles(token)

    # Assert
    assert exception.value.status_code == 403
    assert exception.value.detail == "Missing role."


def test__verify_roles__missing_role_of_missing_client() -> None:
    with pytest.raises(HTTPException) as exception:
        # Arrange
        jwt_auth = JwtAuth(
            AuthConfig("https://authserver.com/auth/realms/projectName", "audienceOne"),
            [
                TokenRole("clientOne", "clientOneRoleOne"),
                TokenRole("clientTwo", "clientTwoRoleTwo"),
                TokenRole("missingClient", "missingRole"),
            ],
            None,
        )

        resource_access: Dict[str, Dict[str, List[str]]] = {
            "clientOne": {"roles": ["clientOneRoleOne", "clientOneRoleTwo"]},
            "clientTwo": {"roles": ["clientTwoRoleOne", "clientTwoRoleTwo"]},
        }

        token: Token = get_token(None, resource_access)

        # Act
        jwt_auth._verify_roles(token)

    # Assert
    assert exception.value.status_code == 403
    assert exception.value.detail == "Missing role."


def test__verify_roles__token_with_empty_roles() -> None:
    with pytest.raises(HTTPException) as exception:
        # Arrange
        jwt_auth = JwtAuth(
            AuthConfig("https://authserver.com/auth/realms/projectName", "audienceOne"),
            [
                TokenRole("clientOne", "clientOneRoleOne"),
                TokenRole("clientTwo", "clientTwoRoleTwo"),
                TokenRole("missingClient", "missingRole"),
            ],
            None,
        )

        token: Token = get_token(None, {})

        # Act
        jwt_auth._verify_roles(token)

    # Assert
    assert exception.value.status_code == 403
    assert exception.value.detail == "Missing role."


def test__verify_roles__token_without_roles() -> None:
    with pytest.raises(HTTPException) as exception:
        # Arrange
        jwt_auth = JwtAuth(
            AuthConfig("https://authserver.com/auth/realms/projectName", "audienceOne"),
            [
                TokenRole("clientOne", "clientOneRoleOne"),
                TokenRole("clientTwo", "clientTwoRoleTwo"),
                TokenRole("missingClient", "missingRole"),
            ],
            None,
        )

        token: Token = get_token(None, None)

        # Act
        jwt_auth._verify_roles(token)

    # Assert
    assert exception.value.status_code == 403
    assert exception.value.detail == "Missing role."



from typing import List, cast

import pytest
from fastapi.openapi.models import OAuth2 as OAuth2Model

from sag_py_auth.jwt_auth import JwtAuth
from sag_py_auth.models import AuthConfig, TokenRole


def test__jwt_auth__init__with_valid_params__verify_flow() -> None:
    # Arrange
    auth_config = AuthConfig("https://authserver.com/auth/realms/projectName", "audienceOne")
    required_roles: List[TokenRole] = [TokenRole("clientOne", "clientOneRoleOne")]
    required_realm_roles: List[str] = ["realmRoleOne"]

    # Act
    jwt = JwtAuth(auth_config, required_roles, required_realm_roles)

    # Assert
    oauth_model: OAuth2Model = cast(OAuth2Model, jwt.model)
    assert jwt.required_roles == required_roles
    assert jwt.required_realm_roles == required_realm_roles
    assert oauth_model.flows.authorizationCode is not None
    assert (
        oauth_model.flows.authorizationCode.authorizationUrl
        == "https://authserver.com/auth/realms/projectName/protocol/openid-connect/auth"
    )
    assert (
        oauth_model.flows.authorizationCode.tokenUrl
        == "https://authserver.com/auth/realms/projectName/protocol/openid-connect/token"
    )


def test__jwt_auth__init__with_invalid_issuer() -> None:
    with pytest.raises(Exception) as exception:
        # Arrange
        auth_config = AuthConfig("malformedUrl", "audienceOne")
        required_roles: List[TokenRole] = [TokenRole("clientOne", "clientOneRoleOne")]
        required_realm_roles: List[str] = ["realmRoleOne"]

        # Act
        JwtAuth(auth_config, required_roles, required_realm_roles)

    # Assert
    assert "Invalid issuer or audience" in str(exception)


def test__jwt_auth__init__with_empty_audience() -> None:
    with pytest.raises(Exception) as exception:
        # Arrange
        auth_config = AuthConfig("https://authserver.com/auth/realms/projectName", "")
        required_roles: List[TokenRole] = [TokenRole("clientOne", "clientOneRoleOne")]
        required_realm_roles: List[str] = ["realmRoleOne"]

        # Act
        JwtAuth(auth_config, required_roles, required_realm_roles)

    # Assert
    assert "Invalid issuer or audience" in str(exception)


def test__jwt_auth__init__with_none_audience() -> None:
    with pytest.raises(Exception) as exception:
        # Arrange
        auth_config = AuthConfig("https://authserver.com/auth/realms/projectName", None)  # type: ignore
        required_roles: List[TokenRole] = [TokenRole("clientOne", "clientOneRoleOne")]
        required_realm_roles: List[str] = ["realmRoleOne"]

        # Act
        JwtAuth(auth_config, required_roles, required_realm_roles)

    # Assert
    assert "Invalid issuer or audience" in str(exception)



import logging
from logging import Logger
from typing import List, NoReturn, Optional

from fastapi.exceptions import HTTPException
from fastapi.security.oauth2 import OAuth2AuthorizationCodeBearer
from starlette.requests import Request
from starlette.status import HTTP_401_UNAUTHORIZED, HTTP_403_FORBIDDEN

from sag_py_auth.auth_context import set_token as auth_context_set_token
from sag_py_auth.models import AuthConfig, Token, TokenRole
from sag_py_auth.token_decoder import verify_and_decode_token
from sag_py_auth.token_types import TokenDict
from sag_py_auth.utils import validate_url

logger: Logger = logging.getLogger(__name__)


class JwtAuth(OAuth2AuthorizationCodeBearer):
    def __init__(
        self,
        auth_config: AuthConfig,
        required_roles: Optional[List[TokenRole]],
        required_realm_roles: Optional[List[str]],
    ) -> None:
        # sourcery skip: raise-specific-error

        if not validate_url(auth_config.issuer) or not auth_config.audience:
            raise Exception("Invalid issuer or audience")

        self.auth_config: AuthConfig = auth_config
        self.required_roles: List[TokenRole] = required_roles or []
        self.required_realm_roles: List[str] = required_realm_roles or []

        super().__init__(
            f"{auth_config.issuer}/protocol/openid-connect/auth",
            f"{auth_config.issuer}/protocol/openid-connect/token",
            scopes={},
        )

    async def __call__(self, request: Request) -> Token:  # type: ignore
        token_string: Optional[str] = await super(JwtAuth, self).__call__(request)
        assert token_string is not None

        token: Token = self._verify_and_decode_token(token_string)
        assert token is not None  # always set if no exception

        self._verify_roles(token)
        self._verify_realm_roles(token)
        auth_context_set_token(token)
        return token

    def _verify_and_decode_token(self, token_string: str) -> Token:  # type: ignore
        try:
            token_dict: TokenDict = verify_and_decode_token(self.auth_config, token_string)
            return Token(token_dict)
        except Exception:
            logger.warning("Invalid auth token", exc_info=True)
            self._raise_auth_error(HTTP_401_UNAUTHORIZED, "Invalid token.")

    def _verify_roles(self, token: Optional[Token]) -> None:
        has_all_roles: bool = token is not None and all(
            token.has_role(requiredRole.client, requiredRole.role) for requiredRole in self.required_roles
        )

        if not has_all_roles:
            logger.warning("User requires roles '%s'", self.required_roles)
            self._raise_auth_error(HTTP_403_FORBIDDEN, "Missing role.")

    def _verify_realm_roles(self, token: Optional[Token]) -> None:
        token_realm_roles: List[str] = token.get_realm_roles() if token is not None else []
        has_all_realm_roles: bool = all(realm_role in token_realm_roles for realm_role in self.required_realm_roles)

        if not has_all_realm_roles:
            logger.warning("User requires realm roles '%s'", self.required_roles)
            self._raise_auth_error(HTTP_403_FORBIDDEN, "Missing realm role.")

    def _raise_auth_error(self, status_code: int, detail: str) -> NoReturn:
        raise HTTPException(status_code=status_code, detail=detail, headers={"WWW-Authenticate": "Bearer"})



from typing import List, Literal, Optional, Tuple

import pytest
from fastapi import Request
from pytest import MonkeyPatch
from starlette.datastructures import Headers

from sag_py_auth.jwt_auth import JwtAuth
from sag_py_auth.models import AuthConfig, Token, TokenRole
from sag_py_auth.token_types import TokenDict

from .helpers import get_token_dict

pytest_plugins: Tuple[Literal["pytest_asyncio"]] = ("pytest_asyncio",)


def verify_and_decode_token_mock(_: AuthConfig, __: str) -> TokenDict:
    return get_token_dict(None, None)


def _verify_roles_mock(_: JwtAuth, token: Optional[Token]) -> None:
    if token:
        token.token_dict["_verify_roles"] = "True"


def _verify_realm_roles_mock(_: JwtAuth, token: Optional[Token]) -> None:
    if token:
        token.token_dict["_verify_realm_roles"] = "True"


def auth_context_set_token_mock(token: Token) -> None:
    if token:
        token.token_dict["auth_context_set_token"] = "True"


@pytest.mark.asyncio
async def test__call__correctly_processes_request(monkeypatch: MonkeyPatch) -> None:
    # Arrange
    auth_config = AuthConfig("https://authserver.com/auth/realms/projectName", "audienceOne")
    required_roles: List[TokenRole] = [TokenRole("clientOne", "clientOneRoleOne")]
    required_realm_roles: List[str] = ["realmRoleOne"]

    monkeypatch.setattr("sag_py_auth.jwt_auth.verify_and_decode_token", verify_and_decode_token_mock)
    monkeypatch.setattr("sag_py_auth.jwt_auth.JwtAuth._verify_roles", _verify_roles_mock)
    monkeypatch.setattr("sag_py_auth.jwt_auth.JwtAuth._verify_realm_roles", _verify_realm_roles_mock)
    monkeypatch.setattr("sag_py_auth.jwt_auth.auth_context_set_token", auth_context_set_token_mock)

    jwt = JwtAuth(auth_config, required_roles, required_realm_roles)

    request: Request = Request(scope={"type": "http"})
    request._headers = Headers({"Authorization": "Bearer validToken"})

    # Act
    actual: Token = await jwt.__call__(request)

    # Assert - Verify that all steps have been executed
    assert actual.get_field_value("typ") == "Bearer"
    assert actual.get_field_value("azp") == "public-project-swagger"
    assert actual.get_field_value("_verify_roles") == "True"
    assert actual.get_field_value("_verify_realm_roles") == "True"
    assert actual.get_field_value("auth_context_set_token") == "True"



from typing import Any, Dict, Optional

import requests
from jose import jwt

from sag_py_auth.models import AuthConfig
from sag_py_auth.token_types import JwkDict, JwksDict, TokenDict

cached_jwk: Optional[JwkDict] = None


def verify_and_decode_token(auth_config: AuthConfig, token_string: str) -> TokenDict:
    """Decode and verify the token

    Returns: The token
    """
    global cached_jwk

    if not cached_jwk:
        cached_jwk = _get_token_jwk(auth_config.issuer, token_string)

    # "decode" also verifies signature, issuer, audience, expiration and more
    token: TokenDict = jwt.decode(
        token=token_string, key=cached_jwk, audience=auth_config.audience, issuer=auth_config.issuer
    )
    return token


def _get_token_jwk(issuer: str, token_string: str) -> JwkDict:
    """Gets the key set sent from the auth provider (idp)
    that belongs to the token in the parameter. The correct
    key set is identified by key id (kid). The kid is part
    of the header information of the token.

    Returns: The key set that belongs to the token
    """

    token_header: Dict[str, Any] = jwt.get_unverified_header(token_string)
    token_key_id: str = token_header["kid"]

    auth_provider_jwks: JwksDict = _get_auth_provider_jwks(issuer)
    return auth_provider_jwks[token_key_id]


def _get_auth_provider_jwks(issuer: str) -> JwksDict:
    """Json web tokens are completely verified on the client side.
    The token is signed by the auth provider (idp) to avoid manipulation.
    To verify if the token is from the expected idp we need to request the
    public key and signing algorithm information from the idp.
    One idp can have multiple json web key sets (jwks). The key set is
    identified by the key id (kid) sent by the server.

    Returns: All key sets of the idp
    """
    jwks_request_url: str = f"{issuer}/protocol/openid-connect/certs"
    jwks_request_headers: Dict[str, str] = {"content-type": "application/json"}
    timeout_seconds = 30
    jwks_response: Dict[str, Any] = requests.get(
        jwks_request_url, headers=jwks_request_headers, timeout=timeout_seconds
    ).json()

    return {jwk["kid"]: jwk for jwk in jwks_response["keys"]}



from logging import LogRecord
from typing import cast

from sag_py_auth.auth_context import set_token as set_token_to_context
from sag_py_auth.models import Token, UserInfoLogRecord
from sag_py_auth.user_name_logging_filter import UserNameLoggingFilter

from .helpers import get_token


def test__get_field_value__with_valid_data() -> None:
    # Arrange
    logging_filter = UserNameLoggingFilter()
    log_entry = LogRecord("LogRecord", 0, "path.py", 100, "A test message", None, None)

    token: Token = get_token(None, None)
    set_token_to_context(token)

    # Act
    logging_filter.filter(log_entry)

    # Assert
    assert log_entry.msg == "A test message"
    assert cast(UserInfoLogRecord, log_entry).user_name == "preferredUsernameValue"
    assert cast(UserInfoLogRecord, log_entry).authorized_party == "public-project-swagger"


def test__get_field_value__with_missing_data() -> None:
    # Arrange
    logging_filter = UserNameLoggingFilter()
    log_entry = LogRecord("LogRecord", 0, "path.py", 100, "A test message", None, None)

    token: Token = get_token(None, None)
    token.token_dict["preferred_username"] = None
    token.token_dict["azp"] = None
    set_token_to_context(token)

    # Act
    logging_filter.filter(log_entry)

    # Assert
    assert log_entry.msg == "A test message"
    assert not hasattr(log_entry, "user_name")
    assert not hasattr(log_entry, "authorized_party")



from logging import LogRecord
from typing import List

from sag_py_auth.token_types import TokenDict


class AuthConfig:
    """Auth configuration used to validate the token"""

    def __init__(self, issuer: str, audience: str) -> None:
        self.issuer: str = issuer
        self.audience: str = audience


class Token:
    """The authentication token"""

    def __init__(self, token_dict: TokenDict) -> None:
        self.token_dict: TokenDict = token_dict

    def get_field_value(self, field_name: str) -> str:
        """Gets the value of a specified token claim field

        Returns: The claim field value
        """
        try:
            return self.token_dict[field_name]
        except KeyError:
            return ""

    def get_roles(self, client: str) -> List[str]:
        """Gets all roles of a specific client

        Returns: The client roles
        """
        try:
            return self.token_dict["resource_access"][client]["roles"]
        except KeyError:
            return []

    def has_role(self, client: str, role_name: str) -> bool:
        """Checks if a specific client of the token has a role

        Returns: True if the client has the role
        """
        roles: List[str] = self.get_roles(client)
        return role_name in roles

    def get_realm_roles(self) -> List[str]:
        """Gets all realm roles

        Returns: The realm roles
        """
        try:
            return self.token_dict["realm_access"]["roles"]
        except KeyError:
            return []

    def has_realm_role(self, role_name: str) -> bool:
        """Checks if the token has a realm role

        Returns: True if the token has the client role
        """
        roles: List[str] = self.get_realm_roles()
        return role_name in roles


class TokenRole:
    """
    Define required token auth roles
    """

    def __init__(self, client: str, role: str) -> None:
        self.client: str = client
        self.role: str = role


class UserInfoLogRecord(LogRecord):
    user_name: str
    authorized_party: str



from typing import Dict, List

from sag_py_auth.models import Token

from .helpers import get_token


def test__token__get_field_value__with_existing_key() -> None:
    # Arrange
    token: Token = get_token(None, None)

    # Act
    actual: str = token.get_field_value("azp")

    # Assert
    assert actual == "public-project-swagger"


def test__token__get_field_value__with_missing_key() -> None:
    # Arrange
    token: Token = get_token(None, None)

    # Act
    actual: str = token.get_field_value("missingKey")

    # Assert
    assert not actual


def test__token__roles() -> None:
    # Arrange
    resource_access: Dict[str, Dict[str, List[str]]] = {
        "clientOne": {"roles": ["clientOneRoleOne", "clientOneRoleTwo"]},
        "clientTwo": {"roles": ["clientTwoRoleOne", "clientTwoRoleTwo"]},
    }

    token: Token = get_token(None, resource_access)

    # Act
    actual_client_one: List[str] = token.get_roles("clientOne")
    actual_missing_client: List[str] = token.get_roles("missingClient")

    actual_has_one: bool = token.has_role("clientOne", "clientOneRoleOne")
    actual_has_missing_role: bool = token.has_role("clientOne", "missingRole")
    actual_has_missing_client: bool = token.has_role("missingClient", "missingRole")

    # Assert
    assert actual_client_one == ["clientOneRoleOne", "clientOneRoleTwo"]
    assert not actual_missing_client
    assert actual_has_one
    assert not actual_has_missing_role
    assert not actual_has_missing_client


def test__token__realm_roles() -> None:
    # Arrange
    realm_access: Dict[str, List[str]] = {"roles": ["realmRoleOne", "realmRoleTwo"]}

    token: Token = get_token(realm_access, None)

    # Act
    actual_realm_roles: List[str] = token.get_realm_roles()

    actual_has_one: bool = token.has_realm_role("realmRoleOne")
    actual_has_missing: bool = token.has_realm_role("missingRole")

    # Assert
    assert actual_realm_roles == ["realmRoleOne", "realmRoleTwo"]
    assert actual_has_one
    assert not actual_has_missing
